<?php

namespace Drupal\taxonomy_access_fix;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\BundlePermissionHandlerTrait;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\taxonomy\VocabularyInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

// @todo Remove in 4.x.
if (!trait_exists('\Drupal\Core\Entity\BundlePermissionHandlerTrait')) {
  class_alias('\Drupal\taxonomy_access_fix\BundlePermissionHandlerFallbackTrait', '\Drupal\Core\Entity\BundlePermissionHandlerTrait');
}

/**
 * Provides additional permissions for entities provided by Taxonomy module.
 */
class TaxonomyAccessFixPermissions implements ContainerInjectionInterface {

  use BundlePermissionHandlerTrait;
  use StringTranslationTrait;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a TaxonomyAccessFixPermissions instance.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static($container->get('entity_type.manager'));
  }

  /**
   * Gets additional permissions for Taxonomy Vocabulary entities.
   *
   * @return array
   *   Permissions array.
   */
  public function getPermissions() {
    $vocabularies = $this->entityTypeManager
      ->getStorage('taxonomy_vocabulary')
      ->loadMultiple();

    return $this->generatePermissions($vocabularies, [
      $this,
      'buildPermissions',
    ]);
  }

  /**
   * Builds additional taxonomy term permissions for a given vocabulary.
   *
   * @param \Drupal\taxonomy\VocabularyInterface $vocabulary
   *   The vocabulary.
   *
   * @return array
   *   An array of permission names and descriptions.
   */
  protected function buildPermissions(VocabularyInterface $vocabulary) {
    $permissions = [];

    // @todo Change to "Vocabulary: Action" in 4.x.
    $permissions['view terms in ' . $vocabulary->id()] = [
      'title' => $this->t('View terms in %vocabulary', [
        '%vocabulary' => $vocabulary->label(),
      ]),
    ];
    $permissions['reorder terms in ' . $vocabulary->id()] = [
      'title' => $this->t('Reorder terms in %vocabulary', [
        '%vocabulary' => $vocabulary->label(),
      ]),
    ];

    return $permissions;
  }

}
